import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

public class OutputPanel extends JPanel {
	private static final double LN2 = Math.log(2);
	private static final int WIDTH = 320;
	private static final int HEIGHT = 640;
	private static final int MINGRIDSIZE = 5;
	private static final int MAXGRIDSIZE = 80;

	private int gridSize = 20;
	private int numCol;
	private int numRow;
	private int startPosition;

    private BufferedImage offImage;
    private Graphics2D offGraphics;

	private boolean[][] tape;
	private int[] states;
	private int[] positions;

	private int numState;

	public OutputPanel() {
		setBackground(Color.white);

		numCol = WIDTH / gridSize;
		numRow = HEIGHT / gridSize;
		startPosition = (TuringMachine.LENGTH - numCol) / 2;
		initTape();
	}

	public void initTape() {
		tape = new boolean[HEIGHT / MINGRIDSIZE][TuringMachine.LENGTH];
		states = new int[HEIGHT / MINGRIDSIZE];
		positions = new int[HEIGHT / MINGRIDSIZE];
		for (int y = 0; y < HEIGHT / MINGRIDSIZE; y++) {
			for (int x = 0; x < TuringMachine.LENGTH; x++)
				tape[y][x] = false;
			positions[y] = -1;
		}
	}

	public synchronized void setZoom(int zoom) {
		gridSize = (int)(MAXGRIDSIZE / Math.exp(LN2 * zoom));
		numCol = WIDTH / gridSize;
		numRow = HEIGHT / gridSize;
		startPosition = (TuringMachine.LENGTH - numCol) / 2;
	}

	public synchronized void setMove(int move) {
		startPosition = (TuringMachine.LENGTH - numCol) / 2 + move / (gridSize / MINGRIDSIZE);
	}

	public void setNumState(int numState) {
		this.numState = numState;
	}

	public void updateTape(boolean value, int valuePosition, int state, int statePosition) {
		for (int y = HEIGHT / MINGRIDSIZE - 1; y > 0; y--) {
			System.arraycopy(tape[y - 1], 0, tape[y], 0, TuringMachine.LENGTH);
			states[y] = states[y - 1];
			positions[y] = positions[y - 1];
		}

		tape[0][valuePosition] = value;
		states[0] = state;
		positions[0] = statePosition;
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
//		paintTape(g, tape);

		if (offImage == null) {
			Dimension d = getSize();
			offImage = new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_RGB);
			offGraphics = (Graphics2D)offImage.getGraphics();
		}
		paintTape(offGraphics, tape);
		g.drawImage(offImage, 0, 0, this);
	}

	private synchronized void paintTape(Graphics g, boolean[][] tape) {
		for (int y = 0; y < numRow; y++)
		{
			for (int x = 0; x < numCol; x++)
			{
				if (tape[y][startPosition + x])
				{
					g.setColor(Color.lightGray);
					g.fillRect(x * gridSize, (numRow - y - 1) * gridSize, gridSize, gridSize);
				}
				else {
					g.setColor(Color.white);
					g.fillRect(x * gridSize, (numRow - y - 1) * gridSize, gridSize, gridSize);
				}
				g.setColor(Color.black);
				g.drawRect(x * gridSize, (numRow - y - 1) * gridSize, gridSize, gridSize);
			}
			if (positions[y] >= 0) {
				paintState(g, (positions[y] - startPosition), y, states[y]);
			}
		}
	}

	private void paintState(Graphics g, int x, int y, int state) {
		int r = gridSize / 2;
		int origX = x * gridSize + r;
		int origY = (numRow - y) * gridSize - r;
		double angle = 2 * Math.PI / numState * (state - 1) - Math.PI / 2;
		g.setColor(Color.red);
		g.drawLine(origX, origY, (int)(origX + r * Math.cos(angle)), (int)(origY + r * Math.sin(angle)));
	}

	public Dimension getPreferredSize() {
		return new Dimension(WIDTH + 1, HEIGHT + 1);
	}

	public Dimension getMaximumSize() {
		return new Dimension(WIDTH + 1, HEIGHT + 1);
	}

	public Dimension getMinimumSize() {
		return new Dimension(WIDTH + 1, HEIGHT + 1);
	}

}