import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.event.*;

public class TuringMachineApplet extends JApplet
                                 implements ActionListener,
                                            ItemListener,
                                            ChangeListener {
	private static final double LN1_5 = Math.log(1.5);

	private JButton start;
	private JButton step;
	private JButton stop;
	private JButton resume;
	private JSlider zoomSlider;
	private JSlider moveSlider;
	private JSlider speedSlider;

	private JComboBox strategies;
	private OutputPanel output;

	private Timer timer;

	private TuringMachine tm;

	public TuringMachineApplet() {
		tm = new TuringMachine();

        timer = new Timer(400, this);
        timer.setInitialDelay(0);
        timer.setCoalesce(true);
	}

	public void init() {
		output = new OutputPanel();

		JPanel strategyPanel = new JPanel();
		strategies = new JComboBox();
		strategies.addItemListener(this);
		strategies.addItem(new ClassFourStrategy());
		strategies.addItem(new ClassThreeStrategy());
		strategies.addItem(new ClassTwoStrategy());
		strategies.addItem(new FourStateBBStrategy());
		strategies.addItem(new ThreeStateBBStrategy());
		strategies.addItem(new TwoStateBBStrategy());
//		strategies.addItem(new DummyStrategy());
		JLabel label = new JLabel("Please select strategy: ");
		strategyPanel.add(label);
		strategyPanel.add(strategies);

		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
		JPanel upperControlPanel = new JPanel();
		JPanel zoomControlPanel = new JPanel();
		JPanel moveControlPanel = new JPanel();
		JPanel speedControlPanel = new JPanel();


		start = new JButton("Start");
		start.addActionListener(this);
		step = new JButton("Step");
		step.addActionListener(this);
		stop = new JButton("Stop");
		stop.addActionListener(this);
		resume = new JButton("Resume");
		resume.addActionListener(this);
		zoomSlider = new JSlider(0, 4, 2);
		zoomSlider.setMajorTickSpacing(1);
		zoomSlider.setPaintTicks(true);
		zoomSlider.setSnapToTicks(true);
		zoomSlider.addChangeListener(this);
		Hashtable zoomLabelTable = new Hashtable();
		zoomLabelTable.put(new Integer(0), new JLabel("Zoom In"));
		zoomLabelTable.put(new Integer(4), new JLabel("Zoom Out"));
		zoomSlider.setLabelTable(zoomLabelTable);
		zoomSlider.setPaintLabels(true);
		moveSlider = new JSlider(-100, 100, 0);
		moveSlider.addChangeListener(this);
		Hashtable moveLabelTable = new Hashtable();
		moveLabelTable.put(new Integer(-100), new JLabel("Move Left"));
		moveLabelTable.put(new Integer(100), new JLabel("Move Right"));
		moveSlider.setLabelTable(moveLabelTable);
		moveSlider.setPaintLabels(true);
		speedSlider = new JSlider(0, 8, 4);
		speedSlider.addChangeListener(this);
		Hashtable speedLabelTable = new Hashtable();
		speedLabelTable.put(new Integer(0), new JLabel("Slow"));
		speedLabelTable.put(new Integer(8), new JLabel("Fast"));
		speedSlider.setLabelTable(speedLabelTable);
		speedSlider.setPaintLabels(true);

/*		controlPanel.add(strategyPanel);
		controlPanel.add(speedSlider);
		controlPanel.add(zoomSlider);
		controlPanel.add(moveSlider);
		controlPanel.add(start);
		controlPanel.add(step);
		controlPanel.add(stop);
		controlPanel.add(resume);
		controlPanel.add(Box.createHorizontalGlue());
		controlPanel.add(Box.createVerticalGlue());
*/
		controlPanel.add(upperControlPanel);
		controlPanel.add(zoomControlPanel);
		controlPanel.add(moveControlPanel);
		controlPanel.add(speedControlPanel);
		upperControlPanel.add(start);
		upperControlPanel.add(step);
		upperControlPanel.add(stop);
		upperControlPanel.add(resume);
		zoomControlPanel.add(zoomSlider);
		moveControlPanel.add(moveSlider);
		speedControlPanel.add(speedSlider);

		JPanel outputContainer = new JPanel();
		outputContainer.setLayout(new BoxLayout(outputContainer, BoxLayout.Y_AXIS));
		outputContainer.add(output);

		Container contentPane = getContentPane();
//		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.X_AXIS));
//		contentPane.add(outputContainer);
//		contentPane.add(controlPanel);
		contentPane.setLayout(new BorderLayout());
		contentPane.add(outputContainer, BorderLayout.CENTER);
		contentPane.add(strategyPanel, BorderLayout.NORTH);
		contentPane.add(controlPanel, BorderLayout.SOUTH);
	}

	/**
	 * Handles button clicks.
	 */
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == start) {
			setPanelEnabled(false);
			output.initTape();
			tm.reset(1, TuringMachine.LENGTH / 2);
			output.updateTape(false, TuringMachine.LENGTH / 2, tm.getCurrState(), tm.getTapeHead());
			output.repaint();
			timer.start();
		}
		else if (source == step) {
			step();
		}
		else if (source == stop) {
			timer.stop();
			setPanelEnabled(true);
		}
		else if (source == resume) {
			setPanelEnabled(false);
			timer.start();
		}
		else if (source == timer) {
			step();
		}
	}

	/**
	 * Handles strategy selection.
	 */
	public void itemStateChanged(ItemEvent e) {
		Strategy str = (Strategy)e.getItem();
		tm.setStrategy(str);
		output.setNumState(str.getNumState());
	}

	public void stateChanged(ChangeEvent e) {
		Object source = e.getSource();
		if (source == speedSlider) {
			int sliderValue = speedSlider.getValue();
			timer.setDelay((int)(500 / Math.exp(LN1_5 * sliderValue)));
		}
		else if (source == zoomSlider) {
			output.setZoom(zoomSlider.getValue());
			output.repaint();
		}
		else if (source == moveSlider) {
			output.setMove(moveSlider.getValue());
			output.repaint();
		}
	}

	private void setPanelEnabled(boolean isEnabled) {
		start.setEnabled(isEnabled);
		step.setEnabled(isEnabled);
		resume.setEnabled(isEnabled);
		strategies.setEnabled(isEnabled);
	}

	private void step() {
		boolean halt = tm.compute();
		output.updateTape(tm.getOldChar(), tm.getOldTapeHead(), tm.getCurrState(), tm.getTapeHead());
		output.repaint();
		if (halt) {
			timer.stop();
			setPanelEnabled(true);
		}
	}
}
